import AuthService from '../service/Auth.js';

class Auth {
  static publicRoutes = [];

  static isPublicRoute(route) {
    return Auth.publicRoutes.some((publicRoute) => route.includes(publicRoute));
  }
  static auth() {
    return async (req, res, next) => {
      if (Auth.isPublicRoute(req.path) || req.method === 'GET') {
        return next();
      }
      const { authorization = '' } = req.headers;
      const basicAuthValidation = await AuthService.isValidBasicAuth(authorization);
      if (basicAuthValidation.authorized) {
        return next();
      }

      res.status(401).send({
        message: 'UNAUTHORIZED',
        data: {
          BasicAuth: basicAuthValidation.reason,
        },
      });
    };
  }
  register(server) {
    server.use(Auth.auth());
  }
}

export default new Auth();
